/* Navigator.java
 *
 * created: Sun Jan 10 1999
 *
 * This file is part of Artemis
 * 
 * Copyright (C) 1998,1999  Genome Research Limited
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Header: /nfs/disk222/yeastpub/Repository/powmap/diana/components/Navigator.java,v 1.12 1999/05/05 10:38:39 kmr Exp $
 */

package diana.components;

import diana.sequence.*;
import diana.*;
import Diana;

import java.awt.*;
import java.awt.event.*;

/**
 *  This component allows the user to navigate around the Entry.  It
 *
 *  @author Kim Rutherford
 *  @version $Id: Navigator.java,v 1.12 1999/05/05 10:38:39 kmr Exp $
 **/

public class Navigator extends Frame {
  /**
   *  Create a new Navigator component.
   *  @param selection_owner The owner of the selection that the commands in
   *    the menu will operate on.
   *  @param goto_event_source The object the we will call makeBaseVisible ()
   *    on.
   *  @param entry_group The EntryGroup object used when searching for
   *    qualifier text in features.
   **/
  public Navigator (final SelectionOwner selection_owner,
                    final GotoEventSource goto_event_source,
                    final EntryGroup entry_group) {
    super ("P.O.W.M.A.P. Navigator");

    this.selection_owner = selection_owner;
    this.entry_group = entry_group;
    this.goto_event_source = goto_event_source;

    final Font default_font = Diana.getOptions ().getFont ();

    GridBagLayout gridbag = new GridBagLayout();
    setLayout (gridbag);

    setFont (default_font);

    GridBagConstraints c = new GridBagConstraints();

    c.fill = GridBagConstraints.HORIZONTAL;
    c.anchor = GridBagConstraints.NORTH;
    c.weighty = 0;

    final CheckboxGroup checkbox_group = new CheckboxGroup ();

    goto_base_button = new Checkbox ("Goto Base:", checkbox_group, true);
    final Panel goto_base_panel = new Panel ();
    goto_base_panel.setLayout (new FlowLayout (FlowLayout.LEFT));
    goto_base_panel.add (goto_base_button);
    c.gridwidth = 2;
    gridbag.setConstraints (goto_base_panel, c);
    add (goto_base_panel);

    goto_base_text = new TextField ("", 18);
    c.gridwidth = GridBagConstraints.REMAINDER;
    gridbag.setConstraints (goto_base_text, c);
    add (goto_base_text);

    goto_base_text.addKeyListener (new KeyAdapter () {
      public void keyTyped(final KeyEvent e) {
        goto_base_button.setState (true);
        if (e.getKeyChar () == '\n') {
//  this sometimes causes an infinite loop in v1.1.7
//          doGoto ();
        }
      }
    });


    goto_text_button =
      new Checkbox ("Goto Feature With This Text:", checkbox_group, true);
    final Panel goto_text_panel = new Panel ();
    goto_text_panel.setLayout (new FlowLayout (FlowLayout.LEFT));
    goto_text_panel.add (goto_text_button);
    c.gridwidth = 2;
    gridbag.setConstraints (goto_text_panel, c);
    add (goto_text_panel);

    goto_feature_text_textfield = new TextField ("", 18);
    c.gridwidth = GridBagConstraints.REMAINDER;
    gridbag.setConstraints (goto_feature_text_textfield, c);
    add (goto_feature_text_textfield);

    goto_feature_text_textfield.addKeyListener (new KeyAdapter () {
      public void keyTyped(final KeyEvent e) {
        goto_text_button.setState (true);
        if (e.getKeyChar () == '\n') {
//  this sometimes causes an infinite loop in v1.1.7
//          doGoto ();
        }
      }
    });


    goto_base_pattern_button =
      new Checkbox ("Goto Base Pattern:", checkbox_group, true);
    final Panel goto_base_pattern_panel = new Panel ();
    goto_base_pattern_panel.setLayout (new FlowLayout (FlowLayout.LEFT));
    goto_base_pattern_panel.add (goto_base_pattern_button);
    c.gridwidth = 2;
    gridbag.setConstraints (goto_base_pattern_button, c);
    add (goto_base_pattern_panel);

    goto_base_pattern_text = new TextField ("", 18);
    c.gridwidth = GridBagConstraints.REMAINDER;
    gridbag.setConstraints (goto_base_pattern_text, c);
    add (goto_base_pattern_text);

    goto_base_pattern_text.addKeyListener (new KeyAdapter () {
      public void keyTyped(final KeyEvent e) {
        goto_base_pattern_button.setState (true);
        if (e.getKeyChar () == '\n') {
//  this sometimes causes an infinite loop in v1.1.7
//          doGoto ();
        }
      }
    });


    checkbox_group.setSelectedCheckbox (goto_base_button);

    final CheckboxGroup start_position_checkbox_group =
      new CheckboxGroup ();

    final Panel start_at_beginning_panel = new Panel ();

    start_at_beginning_button =
      new Checkbox ("Start search at beginning",
                    start_position_checkbox_group,
                    true);

    start_at_beginning_panel.add (start_at_beginning_button);

    final Panel start_at_selection_panel = new Panel ();

    start_at_selection_button =
      new Checkbox ("Start search at selection",
                    start_position_checkbox_group,
                    false);

    start_at_selection_panel.add (start_at_selection_button);


    gridbag.setConstraints (start_at_beginning_panel, c);
    add (start_at_beginning_panel);

    gridbag.setConstraints (start_at_selection_panel, c);
    add (start_at_selection_panel);


    final Button goto_button = new Button ("Goto");

    goto_button.addActionListener (new ActionListener () {
      public void actionPerformed (ActionEvent e) {
//        stopListening ();
        doGoto ();
      }
    });


    final Button close_button = new Button ("Close");

    close_button.addActionListener (new ActionListener () {
      public void actionPerformed (ActionEvent e) {
//        stopListening ();
        Navigator.this.dispose ();
      }
    });


    final Panel close_and_goto_panel = new Panel ();

    close_and_goto_panel.add (goto_button);
    close_and_goto_panel.add (close_button);

    gridbag.setConstraints (close_and_goto_panel, c);
    add (close_and_goto_panel);


    addWindowListener (new WindowAdapter () {
      public void windowClosing (WindowEvent event) {
//        stopListening ();
        Navigator.this.dispose ();
      }
    });


    pack ();

    final Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
    setLocation (new Point ((screen.width - getSize ().width) / 2,
                            (screen.height - getSize ().height) / 2));

    setVisible (true);
  }

  /**
   *  This method will perform the selected goto function.
   **/
  private void doGoto () {
    if (goto_base_button.getState () == true) {
      doGotoBase ();
      return;
    }

    if (goto_base_pattern_button.getState () == true) {
      doGotoBasePattern ();
      return;
    }

    if (goto_text_button.getState () == true) {
      doGotoText ();
      return;
    }
  }

  /**
   *  Go to the base that the user typed in to the goto_base_text TextArea.
   **/
  private void doGotoBase () {
    final String number_string = goto_base_text.getText ().trim ();

    if (number_string.length () == 0) {
      new MessageDialog (this, "you have not entered a number to go to");
      return;
    }

    try {
      final int destination_base =
        Integer.valueOf (number_string).intValue ();

//        System.out.println ("going to: " + destination_base);

      final MarkerRange destination_range =
        goto_event_source.gotoBase (destination_base);

      if (destination_range == null) {
        new MessageDialog (this,
                           "the base is out of range for this sequence");
      } else {
        // success select that base
        selection_owner.getSelection ().setMarkerRange (destination_range);
      }
    } catch (NumberFormatException e) {
      new MessageDialog (this, "Cannot understand this number: " +
                         goto_base_text.getText ());
    }
  }

  /**
   *  Go to the base pattern that the user typed in to the
   *  goto_base_pattern_text TextArea.
   **/
  private void doGotoBasePattern () {
    try {
      final String pattern_string =
        goto_base_pattern_text.getText ().trim ();

      if (pattern_string.length () == 0) {
        new MessageDialog (this, "you have not entered a pattern to go to");
        return;
      }

      final BasePattern pattern = new BasePattern (pattern_string);

      final boolean start_at_base_1 = start_at_beginning_button.getState ();

      start_at_selection_button.setState (true);

      final MarkerRange match_range =
        goto_event_source.gotoBasePattern (pattern,
                                           start_at_base_1);

      if (match_range == null) {
        new MessageDialog (this, "reached the end of sequence");
      } else {
        selection_owner.getSelection ().setMarkerRange (match_range);
      }
    } catch (BasePatternFormatException e) {
      new MessageDialog (this,
                         "Illegal base pattern: " +
                         goto_base_pattern_text.getText ());
    }
  }

  /**
   *  Select the next feature that contains the text given by the user in the
   *  goto_feature_text_textfield TextArea.
   **/
  private void doGotoText () {
    final FeatureVector selected_features =
      selection_owner.getSelection ().getAllFeatures ();

    final int index;

    if (selected_features.size () == 0 ||
        start_at_beginning_button.getState ()) {
      index = -1;
    } else {
      index = entry_group.indexOf (selected_features.elementAt (0));
    }

    final int first_search_feature_index;

    if (index == -1) {
      // nothing was selected so start the search at the first feature
      first_search_feature_index = 0;
    } else {
      first_search_feature_index = index + 1;
    }

    final String search_text = goto_feature_text_textfield.getText ().trim ();

    for (int i = first_search_feature_index ;
         i < entry_group.getAllFeaturesCount () ;
         ++i) {
      final Feature this_feature = entry_group.featureAt (i);

      if (this_feature.containsText (search_text, true)) {
        selection_owner.getSelection ().set (this_feature);
        goto_event_source.gotoBase (this_feature.getFirstBaseMarker ());
        start_at_selection_button.setState (true);
        return;
      }
    }

    selection_owner.getSelection ().clear ();
    start_at_beginning_button.setState (true);

    new MessageDialog (this, "text not found");
  }

  /**
   *  The Checkbox that selects the goto base function.
   **/
  final Checkbox goto_base_button;

  /**
   *  The Checkbox that selects the goto base pattern function.
   **/
  final Checkbox goto_base_pattern_button;

  /**
   *  The Checkbox that selects the goto feature text function.
   **/
  final Checkbox goto_text_button;

  /**
   *  This contains the pattern to search for if the user has selected the
   *  goto base function.
   **/
  final TextField goto_base_text;

  /**
   *  This contains the pattern to search for if the user has selected the
   *  goto base pattern function.
   **/
  final TextField goto_base_pattern_text;

  /**
   *  This contains the pattern to search for if the user has selected the
   *  goto base pattern function.
   **/
  final TextField goto_feature_text_textfield;

  /**
   *  The user selects this Checkbox if the search should start at first base
   *  or first feature (depending on the search type).
   **/
  final Checkbox start_at_beginning_button;

  /**
   *  The user selects this Checkbox if the search should start at the
   *  position of the current selection.
   **/
  final Checkbox start_at_selection_button;

  /**
   *  The GotoEventSource object that was passed to the constructor.
   **/
  final GotoEventSource goto_event_source;

  /**
   *  The EntryGroup object that was passed to the constructor.
   **/
  final EntryGroup entry_group;

  /**
   *  This is the selection owner that was passed to the constructor.
   **/
  final private SelectionOwner selection_owner;
}


