/* CorbaFeature.java
 *
 * created: Wed Feb 10 1999
 *
 * This file is part of Artemis
 * 
 * Copyright (C) 1998,1999  Genome Research Limited
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Header: /nfs/disk222/yeastpub/Repository/powmap/embl/CorbaFeature.java,v 1.9 1999/03/23 11:49:18 kmr Exp $
 **/

package embl;

import StringVector;

import nsdb.NucFeature;
import type.NoResult;

/**
 *  This is a subclass of Feature that can read itself from Corba.
 *
 *  @author Kim Rutherford
 *  @version $Id: CorbaFeature.java,v 1.9 1999/03/23 11:49:18 kmr Exp $
 **/

public class CorbaFeature extends EMBLObject implements Feature {
  /**
   *  Create a new CorbaFeature object.
   *  @param feature_handle The corba handle of the feature
   *  @exception InvalidRelationException Thrown if this Feature cannot contain
   *    one of the qualifiers that was read from the server.
   **/
  public CorbaFeature (final NucFeature feature_handle)
      throws LocationParseException, InvalidKeyException, NoResult,
             InvalidRelationException {
    this.feature_handle = feature_handle;

    final String location_string =
      feature_handle.getLocation ().getLocationString ();

//    System.out.println ("location: " + location_string);

    this.location = new Location (location_string);
    this.key = new Key (feature_handle.getKey ());
    grabQualifiers ();
  }

  /**
   *  Set the value of this object.
   *  @param key The new feature key
   *  @param location The Location object for the new feature
   *  @param qualifiers The qualifiers for the new feature
   *  @exception InvalidRelationException Thrown if this Feature cannot contain
   *    any of the given Qualifier objects.
   **/
  public void set (Key key,
                   Location location,
                   QualifierVector qualifiers)
      throws InvalidRelationException {
    throw new Error ("not implemented");
  }

  /**
   *  Set the owning Entry of this Feature.  Other objects should call
   *  setCorbaEntry () to change the owner.
   *  @param entry The Entry that now owns this Feature.
   **/
  private void setEntry (CorbaEntry entry) {
    this.entry = entry;
  }

  /**
   *  Set the owning CorbaEntry of this Feature.
   *  @param entry The CorbaEntry that now owns this Feature.
   **/
  void setCorbaEntry (final CorbaEntry entry) {
    setEntry (entry);
  }

  /**
   *  Set the key field of this object.
   *  @param key The new feature key
   **/
  public void setKey (Key key) {
    throw new Error ("not implemented");
  }

  /**
   *  Set the location of this object.
   *  @param location The Location object for the new feature
   **/
  public void setLocation (Location location) {
    throw new Error ("not implemented");
  }

  /**
   *  Set the qualifiers of this object discarding all the current qualifiers.
   *  @param qualifiers The qualifiers for the new feature
   *  @exception InvalidRelationException Throw if this Feature cannot contain
   *    any of the given Qualifier objects.
   **/
  public void setQualifiers (QualifierVector qualifiers)
      throws InvalidRelationException  {
    throw new Error ("not implemented");
  }

//   /**
//    *  Add the given Qualifier to this Feature.  If this Feature contains a
//    *  Qualifier with the same name as the new Qualifier it will be replaced.
//    *  @param qualifier The new qualifier to add.
//    *  @exception InvalidRelationException Throw if this Feature cannot contain
//    *    the given Qualifier.
//    **/
//   public void setQualifier (Qualifier qualifier)
//       throws InvalidRelationException;

  /**
   *  Add the values from the given qualifier to the Qualifier object with the
   *  same name in this Feature or otherwise add a copy of the argument.
   *  @param qualifier This object contians name and values to add.
   *  @param values The values to add to the Qualifier.
   *  @return The Qualifier that was changed or created.
   *  @exception InvalidRelationException Thrown if this Feature cannot contain
   *    the given Qualifier.
   **/
  public Qualifier addQualifierValues (Qualifier qualifier)
      throws InvalidRelationException  {
    throw new Error ("not implemented");
  }

  /**
   *  Return the unique identifier of this StreamFeature.
   **/
  long getNumericID () {
    return id;
  }

  /**
   *  Return the key of this feature, as passed to the constructor.
   **/
  public Key getKey () {
    return key;
  }

  /**
   *  Return the Location of this Feature, as passed to the constructor.
   **/
  public Location getLocation () {
    return location;
  }

  /**
   *  Return a QualifierVector object containing the qualifiers for this
   *  feature.  This method does not return a copy of the qualifier vector so
   *  changing the vector will change the feature.
   **/
  public QualifierVector getQualifiers () {
    return qualifiers;
  }

  /**
   *  Return the Qualifier in this Feature with the given name or null if
   *  there no such Qualifier.
   **/
  public Qualifier getQualifierByName (final String name) {
    return getQualifiers ().getQualifierByName (name);
  }

  /**
   *  Return the first base of this feature.
   **/
  public int getFirstBase () {
    return getLocation ().getFirstBase ();
  }

  /**
   *  Return the last base of this feature.
   **/
  public int getLastBase () {
    return getLocation ().getLastBase ();
  }

  /**
   *  Return the Entry object that contains this Feature.
   **/
  public Entry getEntry () {
    return entry;
  }

  /**
   *  Return the reference of a new copy of this Feature.  The new Feature
   *  will not be in any Entry, so it should be explicitly added with a call
   *  to Entry.add ().
   **/
  public Feature copy () {
    throw new Error ("not implemented");
  }

  /**
   *  Read the qualifiers from the given NucFeature, create an embl.Qualifier
   *  for each one and put them in the qualifiers vector.
   **/
  private void grabQualifiers ()
      throws InvalidRelationException {
    qualifiers = new QualifierVector ();

    try {
      final nsdb.NucFeaturePackage.Qualifier [] corba_qualifier_handles =
        feature_handle.getQualifiers ();

//      System.out.println ("feature " + getKey () + " - " +
//                          corba_qualifier_handles.length);

      for (int i = 0 ; i < corba_qualifier_handles.length ; ++i) {
        final nsdb.NucFeaturePackage.Qualifier this_corba_qualifier =
          corba_qualifier_handles[i];

        final StringVector values =
          getQualifierStringValues (this_corba_qualifier);

        if (!Qualifier.isPossibleQualifier (getKey (),
                                            this_corba_qualifier.name)) {
          final String message =
            getKey () + " cannot have " + this_corba_qualifier.name +
            " as a qualifier";
          throw new InvalidRelationException (message);
        }

        if (values.size () == 0) {
          qualifiers.setQualifier (new Qualifier (this_corba_qualifier.name));
        } else {
          qualifiers.setQualifier (new Qualifier (this_corba_qualifier.name,
                                                  values));
        }
      }
    } catch (InvalidRelationException e) {
      System.out.println ("exception reading from corba: " + e);
    } catch (QualifierParseException e) {
      System.out.println ("qualifier parse error reading from corba: " + e);
    } catch (InvalidQualifierException e) {
      System.out.println ("invalid qualifier error while reading " +
                          "from corba: " + e);
    } catch (NoResult e) {
      System.out.println ("unexpected NoResult exception while reading " +
                          "from corba " + e);
    }
  }

  /**
   *  Return (from corba) the values of the given qualifier as String objects.
   **/
  private StringVector
    getQualifierStringValues (nsdb.NucFeaturePackage.Qualifier qualifier) {

    final nsdb.NucFeaturePackage.QualifierValue_u [] corba_values =
      qualifier.values;

    final StringVector return_array = new StringVector ();

    for (int value_index = 0 ;
         value_index < corba_values.length ;
         ++value_index) {

      final nsdb.NucFeaturePackage.QualifierValue_u corba_value =
        corba_values[value_index];

      final String this_value_string;
      
      try {
        this_value_string = String.valueOf (corba_value.integer ());
      } catch (org.omg.CORBA.BAD_OPERATION e1) {
        try {
          this_value_string = String.valueOf (corba_value.text ());
        } catch (org.omg.CORBA.BAD_OPERATION e2) {
          try {
            this_value_string = String.valueOf (corba_value.db_xref ());
          } catch (org.omg.CORBA.BAD_OPERATION e3) {
            try {
              this_value_string = String.valueOf (corba_value.real ());
            } catch (org.omg.CORBA.BAD_OPERATION e4) {
              try {
                this_value_string =
                  String.valueOf (corba_value.translation_exception ());
              } catch (org.omg.CORBA.BAD_OPERATION e5) {
                try {
                  this_value_string =
                    String.valueOf (corba_value.codon_translation ());
                } catch (org.omg.CORBA.BAD_OPERATION e6) {
                  try {
                    this_value_string =
                      String.valueOf (corba_value.anti_codon ());
                  } catch (org.omg.CORBA.BAD_OPERATION e7) {
                    continue;
                  }
                }
              }
            }
          }
        }
      }

      return_array.add (this_value_string);
    }

    return return_array;
  }

  /**
   *  The NucFeature object that was passed to the constructor.
   **/
  private NucFeature feature_handle;

  /**
   *  The Location of this Feature, set by the constructor.
   **/
  private Location location;

  /**
   *  The Key of this Feature, set by the constructor.
   **/
  private Key key;

  /**
   *  The qualifiers of this Feature, set by the first call to the
   *  getQualifiers () method.
   **/
  private QualifierVector qualifiers = null;

  /**
   *  The CorbaEntry object that contains this Feature as passed to the
   *  constructor.
   **/
  private CorbaEntry entry;

  /**
   *  A unique identifier for this feature.
   **/
  private final long id = id_counter++;

  /**
   *  This is incremented each time a constructor is called.
   **/
  private static long id_counter = 0;
}
